package com.ninegag.android.library.upload.editor

import android.annotation.SuppressLint
import android.app.Activity
import android.app.Activity.RESULT_OK
import android.app.AlertDialog
import android.app.ProgressDialog
import android.content.Intent
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.graphics.Typeface
import android.net.Uri
import android.os.Bundle
import android.os.Environment
import android.provider.MediaStore
import android.util.Log
import android.view.KeyEvent
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.WindowManager
import android.widget.Toast
import androidx.annotation.NonNull
import androidx.appcompat.app.AppCompatActivity
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.LinearLayoutManager
import com.google.android.material.snackbar.Snackbar
import com.ninegag.android.library.upload.BaseUploadSourceActivity
import com.ninegag.android.library.upload.R
import com.ninegag.android.library.upload.editor.tools.EditingToolsAdapter
import com.ninegag.android.library.upload.editor.tools.ToolType
import com.ninegag.android.library.upload.showAsGagStyle
import com.under9.android.lib.eventbus.RxBus
import com.under9.android.lib.util.ThreadUtil
import com.under9.android.lib.widget.SystemUIColorRestorer
import com.under9.android.lib.widget.inlinecomposer.activity.UploadSourceActivity
import ja.burhanrashid52.photoeditor.OnPhotoEditorListener
import ja.burhanrashid52.photoeditor.PhotoEditor
import ja.burhanrashid52.photoeditor.SaveSettings
import ja.burhanrashid52.photoeditor.TextStyleBuilder
import ja.burhanrashid52.photoeditor.ViewType
import kotlinx.android.synthetic.main.fragment_media_editor.*
import timber.log.Timber
import java.io.File
import java.io.IOException

class MediaEditorFragment: Fragment(), OnPhotoEditorListener,
        PropertiesBSFragment.Properties,
        EditingToolsAdapter.OnItemSelected{

    companion object {
        const val REQ_CODE_EDIT_MEDIA = 1111
        const val KET_UPDATED_TMP_FILE = "updated_tmp_file"
        const val KEY_STICKER_IDS = "sticker_ids"
        const val KEY_TEXT_LEN = "text_len"
        const val KEY_BRUSH_USED = "brush_used"
        const val KEY_RUBBER_USED = "rubber_used"
        const val KEY_UNDO_USED = "undo_used"
        const val KEY_IMAGE_EDITED = "image_edited_media_editor"
        const val VIEW_ADD_LIMIT = 15
    }

    private val CAMERA_REQUEST = 52
    private val PICK_REQUEST = 53
    private lateinit var photoEditor: PhotoEditor
    private lateinit var brushSheet: PropertiesBSFragment
    private lateinit var stickerSheet: StickerBSFragment
    private lateinit var progressDialog: ProgressDialog
    private lateinit var systemUIRestorer: SystemUIColorRestorer
    private val usedStickerIdList = ArrayList<StickerId>()
    // Which use it to set to the tag view
    private var latestStickerId = ""
    private var totalAddedView = 0
    private var totalAddedTextLength = 0
    private var usedBrush = false
    private var usedRubber = false
    private var usedUndo = false

    override fun onStart() {
        super.onStart()
        (context as Activity).window.setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN)
        RxBus.register(this)
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        return inflater.inflate(R.layout.fragment_media_editor, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        systemUIRestorer = SystemUIColorRestorer(getContext()!!, (getContext() as Activity).window)
        systemUIRestorer.darkenUI()

        brushSheet = PropertiesBSFragment()
        stickerSheet = StickerBSFragment()
        stickerSheet.setStickerListener { stickerId, bitmap ->
            ThreadUtil.mainHandler().post {
                if (totalAddedView < VIEW_ADD_LIMIT) {
                    latestStickerId = stickerId
                    if (!usedStickerIdList.contains(stickerId)) {
                        usedStickerIdList.add(stickerId)
                    }
                    photoEditor.addImage(bitmap)
                } else {
                    context?.let {
                        showSnackbar(it.getString(R.string.edit_view_limit_exceed))
                    }
                }
            }

        }
        brushSheet.setPropertiesChangeListener(this)

        val llmTools = LinearLayoutManager(context, LinearLayoutManager.HORIZONTAL, false)
        rvTools.layoutManager = llmTools
        rvTools.adapter = EditingToolsAdapter(this, context!!)

        photoEditor = PhotoEditor.Builder(context, photoEditorView)
                .setPinchTextScalable(true) // set flag to make text scalable when pinch
                .build() // build photo editor sdk

        photoEditor.setOnPhotoEditorListener(this)
        loadImage()

        saveBtn.setOnClickListener {
            Timber.d("Save image by clicking save button")
            saveImage()
        }

        closeBtn.setOnClickListener {
            showSaveDialog()
        }

        view.isFocusableInTouchMode = true
        view.requestFocus()
        view.setOnKeyListener { _, keyCode, event ->
            if(keyCode == KeyEvent.KEYCODE_BACK && event.action == KeyEvent.ACTION_DOWN) {
                showSaveDialog()
                true
            } else {
                false
            }
        }
    }

    override fun onEditTextChangeListener(rootView: View, text: String, colorCode: Int) {
        totalAddedTextLength -= text.length
        val textEditorDialogFragment = TextEditorDialogFragment.show(context as AppCompatActivity, text, colorCode)
        textEditorDialogFragment.setOnTextEditorListener { inputText, colorCode ->
            val styleBuilder = TextStyleBuilder()
            styleBuilder.withTextColor(colorCode)
            totalAddedTextLength += inputText.length
            photoEditor.editText(rootView, inputText, styleBuilder)
            // update the latest word limit
        }
    }

    override fun onAddViewListener(viewType: ViewType, numberOfAddedViews: Int) {
        // filter, image filter, brush drawing view
        val persistNumView = 3
        if (viewType == ViewType.IMAGE || viewType == ViewType.TEXT) {
            ++totalAddedView
        }
        if (viewType == ViewType.BRUSH_DRAWING) {
            usedBrush = true
        }
        if (viewType == ViewType.IMAGE) {
            try {
                photoEditorView.getChildAt(numberOfAddedViews + persistNumView - 1).tag = latestStickerId
            } catch (e: Exception) {
                Timber.e(e)
            }

        }
    }

    override fun onRemoveViewListener(viewType: ViewType, numberOfAddedViews: Int, removedView: View?) {
        if (viewType == ViewType.IMAGE || viewType == ViewType.TEXT) {
            --totalAddedView
        }
        Timber.d("onRemoveViewListener() called with: viewType = [$viewType], numberOfAddedViews = [$numberOfAddedViews]")
        if (viewType == ViewType.IMAGE && removedView != null) {
            val tag = removedView.tag

            for (i in 0 until usedStickerIdList.size) {
                if (usedStickerIdList[i] == tag) {
                    Timber.d("remove at $i")
                    usedStickerIdList.removeAt(i)
                    break
                }
            }
        }
    }

    override fun onStartViewChangeListener(viewType: ViewType) {
        Timber.d("onStartViewChangeListener() called with: viewType = [$viewType]")
    }

    override fun onStopViewChangeListener(viewType: ViewType) {
        Timber.d("onStopViewChangeListener() called with: viewType = [$viewType]")
    }

    private fun loadImage() {
//        photoEditor.clearAllViews()
        val path = (context as Activity).intent.getStringExtra(BaseUploadSourceActivity.KEY_TMP_PATH)
        val imgFile = File(path)
        if(imgFile.exists()) {
            val bitmap = BitmapFactory.decodeFile(imgFile.absolutePath)
            photoEditorView.source?.setImageBitmap(bitmap)
        }
    }

    @SuppressLint("MissingPermission")
    private fun saveImage() {
        showLoading("Saving...")
        val sfc = UploadSourceActivity.getSFC()
        sfc.init(activity!!.applicationContext)
        val target = sfc.externalImagePathForPostID(context, System.currentTimeMillis().toString(), "png")

        val file = File(target)
        try {
            file.createNewFile()

            val saveSettings = SaveSettings.Builder()
                    .setClearViewsEnabled(true)
                    .setTransparencyEnabled(true)
                    .build()

            photoEditor.saveAsFile(file.absolutePath, saveSettings, object : PhotoEditor.OnSaveListener {
                override fun onSuccess(@NonNull imagePath: String) {
                    hideLoading()
                    showSnackbar("Image Saved Successfully")

                    photoEditorView!!.source.setImageURI(Uri.fromFile(File(imagePath)))

                    val intent = (context as Activity).intent
                    with(intent) {
                        putExtra(KET_UPDATED_TMP_FILE, file.absoluteFile.toString())
                        val ids = convertStickerIdsString()
                        putExtra(KEY_IMAGE_EDITED, true)
                        putExtra(KEY_STICKER_IDS, ids)
                        putExtra(KEY_TEXT_LEN, totalAddedTextLength)
                        putExtra(KEY_BRUSH_USED, usedBrush)
                        putExtra(KEY_RUBBER_USED, usedRubber)
                        putExtra(KEY_UNDO_USED, usedUndo)
                    }

                    (context as Activity).setResult(RESULT_OK, intent)
                    (context as Activity).finish()
                }

                override fun onFailure(@NonNull exception: Exception) {
                    hideLoading()
                    showSnackbar("Failed to save Image")
                }
            })
        } catch (e: IOException) {
            e.printStackTrace()
            hideLoading()
            e.message?.let { showSnackbar(it) }
        }
    }

    private fun convertStickerIdsString(): String {
        var idStr = ""
        usedStickerIdList.forEach {
            idStr += "$it,"
        }
        if (idStr != "") {
            idStr = idStr.substring(0, idStr.length - 1)
        }
        return idStr
    }

    override fun onColorChanged(colorCode: Int) {
        photoEditor.setBrushColor(colorCode)
    }

    override fun onOpacityChanged(opacity: Int) {
        photoEditor.setOpacity(opacity)
    }

    override fun onBrushSizeChanged(brushSize: Int) {
        photoEditor.setBrushSize(brushSize.toFloat())
    }

    private fun showSaveDialog() {
        val builder = AlertDialog.Builder(context)
        builder.setMessage("Are you want to exit without saving image ?")
        builder.setPositiveButton("Save") { _, _ ->
            Timber.d("Save image by clicking dialog")
            saveImage()
        }
        builder.setNegativeButton("Cancel") {
            dialog, _ -> dialog.dismiss()
        }

        builder.setNeutralButton("Discard") { _, _ ->
            Timber.d("Discard edit")
            val intent = (context as Activity).intent
            intent.putExtra(KEY_IMAGE_EDITED, false)
            (context as Activity).setResult(RESULT_OK, intent)
            (context as Activity).finish()
        }
        builder.create().show()

    }

    override fun onToolSelected(toolType: ToolType) {
        when (toolType) {
            ToolType.BRUSH -> {
                photoEditor.setBrushDrawingMode(true)
                fragmentManager?.let { brushSheet.show(it, brushSheet.tag) }
            }
            ToolType.TEXT -> {
                val textEditorDialogFragment = TextEditorDialogFragment.show(context as AppCompatActivity)
                textEditorDialogFragment.setOnTextEditorListener { inputText, colorCode ->
                    val styleBuilder = TextStyleBuilder()
                    styleBuilder.withTextColor(colorCode)
                    photoEditor.addText(inputText, styleBuilder)
                    totalAddedTextLength += inputText.length
                }
            }
            ToolType.ERASER -> {
                usedRubber = true
                photoEditor.brushEraser()
            }
            ToolType.STICKER -> fragmentManager?.let { stickerSheet.show(it, stickerSheet.tag) }
            ToolType.UNDO -> {
                usedUndo = true
                photoEditor.undo()
            }

        }
    }


    fun onBackPressed() {
        showSaveDialog()
    }

    protected fun showLoading(@NonNull message: String) {
        progressDialog = ProgressDialog(context)
        progressDialog.setMessage(message)
        progressDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER)
        progressDialog.setCancelable(false)
        progressDialog.show()
    }

    protected fun hideLoading() {
        progressDialog.dismiss()
    }

    protected fun showSnackbar(@NonNull message: String) {
        val view = (context as Activity).findViewById<View>(android.R.id.content)
        if (view != null) {
            val sb = Snackbar.make(view, message, Snackbar.LENGTH_SHORT)
            sb.showAsGagStyle(context!!)
        } else {
            Toast.makeText(context as Activity, message, Toast.LENGTH_SHORT).show()
        }
    }

    override fun onDestroyView() {
        super.onDestroyView()
        RxBus.unregister(this)
        systemUIRestorer.restoreColor()
    }
}
