package com.ninegag.android.library.upload.editor

import android.annotation.SuppressLint
import android.app.Dialog
import android.content.Context
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.os.Bundle
import android.view.View
import androidx.coordinatorlayout.widget.CoordinatorLayout
import androidx.recyclerview.widget.GridLayoutManager
import androidx.recyclerview.widget.RecyclerView

import com.google.android.material.bottomsheet.BottomSheetBehavior
import com.google.android.material.bottomsheet.BottomSheetDialogFragment
import com.ninegag.android.library.upload.R
import com.ninegag.android.library.upload.editor.tools.StickerAdapter

import java.io.IOException
import java.util.ArrayList
import android.graphics.drawable.Drawable
import android.content.res.AssetManager
import android.util.Log
import timber.log.Timber
import com.google.gson.Gson
import com.google.gson.GsonBuilder
import com.google.gson.reflect.TypeToken
import java.nio.charset.Charset


class StickerBSFragment : BottomSheetDialogFragment() {

    private lateinit var stickerList: ArrayList<Sticker>

    private lateinit var stickerListener: StickerListener

    private val mBottomSheetBehaviorCallback = object : BottomSheetBehavior.BottomSheetCallback() {

        override fun onStateChanged(bottomSheet: View, newState: Int) {
            if (newState == BottomSheetBehavior.STATE_HIDDEN) {
                dismiss()
            }
        }

        override fun onSlide(bottomSheet: View, slideOffset: Float) {}
    }

    fun setStickerListener(stickerListener: StickerListener) {
        this.stickerListener = stickerListener
    }

    @SuppressLint("RestrictedApi")
    override fun setupDialog(dialog: Dialog, style: Int) {
        super.setupDialog(dialog, style)
        val contentView = View.inflate(context, R.layout.fragment_bottom_sticker_emoji_dialog, null)
        dialog.setContentView(contentView)
        val params = (contentView.parent as View).layoutParams as CoordinatorLayout.LayoutParams
        val behavior = params.behavior

        if (behavior != null && behavior is BottomSheetBehavior<*>) {
            behavior.setBottomSheetCallback(mBottomSheetBehaviorCallback)
        }
        (contentView.parent as View).setBackgroundColor(resources.getColor(android.R.color.transparent))
        val rvStickers = contentView.findViewById<RecyclerView>(R.id.rvStickers)

        val gridLayoutManager = GridLayoutManager(activity, 3)
        rvStickers.layoutManager = gridLayoutManager

        if (!::stickerList.isInitialized) {
            stickerList = ArrayList()
            initSticker()
        }
        val stickerAdapter = StickerAdapter(context!!, stickerList, stickerListener, dismissCallback = {
            dismiss()
        })
        rvStickers.adapter = stickerAdapter
    }

    private fun initSticker() {
        try {
//            val assetManager = context!!.assets
//            val images = assetManager.list("v1")
//            // the loop read all image in dogs folder and  aa
//            images?.forEach {
//                val stickerId = it.substring(0, it.lastIndexOf('.'))
//                getBitmapFromAsset(context!!, "v1/$it")?.let { bitmap ->
//                    stickerList.add(Pair(stickerId, bitmap))
//                }
//            }
            val jsonString = loadJSONFromAsset()
            if (jsonString != null) {
                val type = object : TypeToken<ArrayList<Sticker>>() {}.type
                val b = GsonBuilder()
                val gson = b.create()
                stickerList = gson.fromJson(jsonString, type)
            }
        } catch (e: IOException) {
            Timber.e(e)
        }
    }

    private fun loadJSONFromAsset(): String? {
        var json: String? = null
        try {
            val inputStream = activity!!.assets.open("v0_stickers.json")
            val size = inputStream.available()
            val buffer = ByteArray(size)
            inputStream.read(buffer)
            inputStream.close()
            val charset = Charsets.UTF_8
            json = String(buffer, charset)
        } catch (ex: IOException) {
            ex.printStackTrace()
            return null
        }

        return json
    }
    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

    }

    private fun getBitmapFromAsset(context: Context, strName: String): Bitmap? {
        return try {
            val inputStream = context.assets.open(strName)
            BitmapFactory.decodeStream(inputStream)
        } catch (e: IOException) {
            e.printStackTrace()
            null
        }
    }

}

typealias DismissCallback = () -> Unit
typealias StickerListener = (StickerId, Bitmap) -> Unit
typealias StickerId = String