package com.ninegag.android.library.upload.editor.tools

import android.content.Context
import android.content.res.Resources
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import androidx.recyclerview.widget.RecyclerView

import com.ninegag.android.library.upload.R
import com.under9.android.lib.util.UiUtil

import java.util.ArrayList

/**
 * @author [Burhanuddin Rashid](https://github.com/burhanrashid52)
 * @version 0.1.2
 * @since 5/23/2018
 */
class EditingToolsAdapter(private val onItemSelected: OnItemSelected, context: Context) : RecyclerView.Adapter<EditingToolsAdapter.ViewHolder>() {

    private val toollist = ArrayList<ToolModel>()
    private val elementWidth: Int
    // minimum width defined in the design which is 75dp
    private var minimumElementWidth = UiUtil.dpToPx(context, 75)

    init {
        toollist.add(ToolModel("Undo", R.drawable.ic_undo, ToolType.UNDO))
        toollist.add(ToolModel("Eraser", R.drawable.ic_eraser, ToolType.ERASER))
        toollist.add(ToolModel("Brush", R.drawable.ic_pen, ToolType.BRUSH))
        toollist.add(ToolModel("Text", R.drawable.ic_text, ToolType.TEXT))
        toollist.add(ToolModel("Sticker", R.drawable.ic_sticker, ToolType.STICKER))

        val screenW = Resources.getSystem().displayMetrics.widthPixels
        elementWidth = screenW / 5
    }

    interface OnItemSelected {
        fun onToolSelected(toolType: ToolType)
    }


    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
        val view = LayoutInflater.from(parent.context)
                .inflate(R.layout.row_editor_tool, parent, false)

        // implementation of the element fill the width
        // if the element not going go to fill, should make the list scroll able
        if (elementWidth > minimumElementWidth) view.layoutParams.width = elementWidth

        return ViewHolder(onItemSelected, toollist, view)
    }

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        val item = toollist[position]
        holder.imgToolIcon.setImageResource(item.toolIcon)
    }

    override fun getItemCount(): Int {
        return toollist.size
    }

    class ViewHolder(private val onItemSelected: OnItemSelected,
                     private val toolList: ArrayList<ToolModel>,
                     itemView: View) : RecyclerView.ViewHolder(itemView) {

        var imgToolIcon: ImageView = itemView.findViewById(R.id.imgToolIcon)

        init {
            itemView.setOnClickListener {
                onItemSelected.onToolSelected(toolList[adapterPosition].toolType)
            }
        }
    }
}

data class ToolModel(val toolName: String,
                     val toolIcon: Int,
                     val toolType: ToolType)

