package com.ninegag.android.library.upload.editor.tools

import android.content.Context
import android.graphics.Bitmap
import android.graphics.drawable.BitmapDrawable
import android.net.Uri
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import androidx.recyclerview.widget.RecyclerView
import com.facebook.common.references.CloseableReference
import com.facebook.datasource.DataSource
import com.facebook.drawee.backends.pipeline.Fresco
import com.facebook.drawee.view.SimpleDraweeView
import com.facebook.imagepipeline.common.Priority
import com.facebook.imagepipeline.common.ResizeOptions
import com.facebook.imagepipeline.datasource.BaseBitmapDataSubscriber
import com.facebook.imagepipeline.image.CloseableImage
import com.facebook.imagepipeline.request.ImageRequestBuilder
import com.ninegag.android.library.upload.R
import com.ninegag.android.library.upload.editor.DismissCallback
import com.ninegag.android.library.upload.editor.Sticker
import com.ninegag.android.library.upload.editor.StickerId
import com.ninegag.android.library.upload.editor.StickerListener
import com.under9.android.lib.util.CallerThreadExecutor
import com.under9.android.lib.util.UiUtil
import com.under9.android.lib.widget.image.frescoprocessor.SquareCroppingPostprocessor
import java.util.ArrayList

class StickerAdapter(context: Context,
                     private val stickerList: ArrayList<Sticker>,
                     private val stickerListener: StickerListener,
                     private val dismissCallback: DismissCallback) : RecyclerView.Adapter<StickerAdapter.ViewHolder>() {

    private val resizeOptions: ResizeOptions = ResizeOptions(UiUtil.dpToPx(context, 96), UiUtil.dpToPx(context, 96))

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
        val view = LayoutInflater.from(parent.context).inflate(R.layout.row_sticker, parent, false)
        return ViewHolder(view, stickerList, stickerListener, dismissCallback)
    }

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        val request = ImageRequestBuilder.newBuilderWithSource(Uri.parse(stickerList[position].url))
                .setResizeOptions(resizeOptions)
                .setRequestPriority(Priority.LOW)
                .build()

        val builder = Fresco.newDraweeControllerBuilder()
                .setImageRequest(request)
                .setOldController(holder.imgSticker.controller)
        holder.imgSticker.controller = builder.build()
    }

    override fun getItemCount(): Int {
        return stickerList.size
    }

    class ViewHolder(itemView: View,
                     private val stickerList: ArrayList<Sticker>,
                     private val stickerListener: StickerListener,
                     private val dismissCallback: DismissCallback) : RecyclerView.ViewHolder(itemView) {
        var imgSticker: SimpleDraweeView = itemView.findViewById(R.id.imgSticker)

        init {
            itemView.setOnClickListener {
                val request = ImageRequestBuilder.newBuilderWithSource(Uri.parse(stickerList[adapterPosition].url))


                Fresco.getImagePipeline().fetchDecodedImage(request.build(), CallerThreadExecutor.getInstance())
                        .subscribe(object: BaseBitmapDataSubscriber() {
                            override fun onFailureImpl(dataSource: DataSource<CloseableReference<CloseableImage>>?) {

                            }

                            override fun onNewResultImpl(bitmap: Bitmap?) {
                                bitmap?.let {
                                    // This is to avoid: java.lang.RuntimeException: Canvas: trying to use a recycled bitmap android.graphics.Bitmap
                                    // The reason may be that Fresco recycled this Bitmap internally, so we have to clone a new bitmap here, to isolate it from Fresco
                                    val clone = Bitmap.createBitmap(it)
                                    stickerListener.invoke(stickerList[adapterPosition].stickerId, clone)
                                    dismissCallback.invoke()
                                }
                            }

                        }, CallerThreadExecutor.getInstance())

            }
        }
    }
}
