package com.ninegag.android.library.upload.media.processor

import android.graphics.Bitmap
import android.graphics.BitmapFactory

import com.ninegag.android.library.upload.Constant
import com.ninegag.android.library.upload.media.validator.BaseMediaValidator
import com.ninegag.android.library.upload.model.MediaMeta
import com.under9.android.lib.util.BitmapUtil

import java.io.File
import java.io.FileOutputStream
import java.io.IOException

/**
 * Created by raymond on 9/5/2017.
 */

class ImageProcessor(saveMediaCallback: SaveMediaCallback,
                     mediaValidator: BaseMediaValidator,
                     private val shouldCompress: Boolean = true) : BaseProcessor(saveMediaCallback, mediaValidator) {

    override fun getMediaMeta(f: File): MediaMeta {
        val opts = BitmapUtil.getBitmapBound(f.absolutePath)
        var width = 0
        var height = 0

        if (opts != null) {
            width = opts.outWidth
            height = opts.outHeight
        }

        return MediaMeta.newBuilder(MediaMeta.MEDIA_TYPE_IMAGE)
                .filePath(f.absolutePath)
                .fileSize(f.length())
                .widthAndHeight(width, height)
                .build()
    }

    @Throws(IOException::class, NullPointerException::class)
    override fun saveMedia(mediaMeta: MediaMeta, tmpFileLocation: String) {
        val fos = FileOutputStream(tmpFileLocation)
        val bitmap = BitmapUtil.smartDecodeBitmap(mediaMeta.filePath.replace("%20".toRegex(), " "), Constant.IMAGE_SIDE_LIMIT)
        if (shouldCompress) {
            bitmap!!.compress(Bitmap.CompressFormat.JPEG, Constant.JPEG_QUALITY, fos)
        } else {
            bitmap!!.compress(Bitmap.CompressFormat.JPEG, Constant.ORIGINAL_QUALITY, fos)
        }
        fos.close()
    }
}
