package com.under9.android.comments.adapter

import android.os.Bundle
import android.view.View
import com.under9.android.comments.model.wrapper.CommentItemWrapperInterface
import com.under9.android.commentsystem.R
import com.under9.android.lib.widget.uiv3.UniversalImageView
import com.under9.android.lib.widget.uiv3.adapter.UIVAdapter
import timber.log.Timber

/**
 * Interface actions that can perform on a specific comment item
 */
interface CommentItemAction {
    fun onItemReply(position: Int, commentWrapper: CommentItemWrapperInterface, prefill: String, suppData: Bundle?)  // db comment id, with extra params including the comment id which user actually tap on
    fun onItemMore(position: Int, commentWrapper: CommentItemWrapperInterface, username: String)
    fun onItemLike(position: Int, commentWrapper: CommentItemWrapperInterface): Boolean
    fun onItemDislike(position: Int, commentWrapper: CommentItemWrapperInterface): Boolean
    fun onItemUnlike(position: Int, commentWrapper: CommentItemWrapperInterface): Boolean
    fun onAuthorNameClick(authorName: String, commentWrapper: CommentItemWrapperInterface)
    fun onAuthorAvatarClick(authorName: String)
    fun onMentionedUserClick(username: String, accountId: String)
    fun onLongClickMedia(position: Int, commentWrapper: CommentItemWrapperInterface): Boolean
    fun onItemOpenOverlay(commentWrapper: CommentItemWrapperInterface, uivAdapter: UIVAdapter)
    fun onItemReport(position: Int, commentWrapper: CommentItemWrapperInterface)
    fun onItemReportAccepted(commentWrapper: CommentItemWrapperInterface)
    fun onItemHide(position: Int, commentWrapper: CommentItemWrapperInterface)
    fun onItemDelete(position: Int, commentWrapper: CommentItemWrapperInterface)
    fun onItemDeleteAccpeted(position: Int, commentWrapper: CommentItemWrapperInterface)
    fun onItemCopy(commentWrapper: CommentItemWrapperInterface)
    fun onItemShare(commentWrapper: CommentItemWrapperInterface)
    fun onItemProBadge(commentWrapper: CommentItemWrapperInterface)
    fun onRefresh(position: Int, commentWrapper: CommentItemWrapperInterface)
    fun onItemUrl(commentItemWrapper: CommentItemWrapperInterface, url: String)
    fun onOpenThread(commentItemWrapperInterface: CommentItemWrapperInterface)

    fun onImageClick(view: View, adapter: UIVAdapter, uiv: UniversalImageView)
    fun onImageLongClick(view: View, adapter: UIVAdapter, uiv: UniversalImageView)
    fun onLoadMoreContainerClick(position: Int, commentWrapper: CommentItemWrapperInterface)
    fun onLoadPrevContainerClick(position: Int, commentWrapper: CommentItemWrapperInterface)
    fun onItemMediaSave(position: Int, commentWrapper: CommentItemWrapperInterface)

    /**
     * // FIXME: 2019-08-23 Not all actions are listed here for time reason...
     * // Only list those need to login to perform
     */
    companion object {
        const val ACTION_TYPE_NONE = -1 // Nothing...
        const val ACTION_TYPE_LIKE = 0
        const val ACTION_TYPE_UNLIKE = 1
        const val ACTION_TYPE_DISLIKE = 2
        val ACTION_TYPE_REPORT = R.id.action_report_comment
        val ACTION_TYPE_DELETE = R.id.action_delete_comment
        const val ACTION_TYPE_DELETE_ACCEPTED = 5
        val ACTION_TYPE_COPY_COMMENT_LINK = R.id.action_copy_comment_link
        val ACTION_TYPE_COPY_COMMENT = R.id.action_copy_comment

        const val ACTION_TYPE_REPORT_ACCEPTED = 8
        const val ACTION_TYPE_REPLY = 9
        const val ACTION_TYPE_VIEW_ADDED_COMMENT = 10
        val ACTION_TYPE_SAVE_TO_GALLERY = R.id.action_save_gallery
    }
}

/**
 * Just a dummy base class, which implements the [CommentItemAction] interface
 */
open class BaseCommentListItemAction: CommentItemAction {
    private val TAG = "BCIActionHandler"
    
    override fun onImageClick(view: View, adapter: UIVAdapter, uiv: UniversalImageView) {
        Timber.tag(TAG).d("onImageClick $view, adapter=$adapter, uiv=$uiv")
    }

    override fun onImageLongClick(view: View, adapter: UIVAdapter, uiv: UniversalImageView) {
        Timber.tag(TAG).d("onImageLongClick $view, adapter=$adapter, uiv=$uiv")
    }

    override fun onItemReply(position: Int, commentWrapper: CommentItemWrapperInterface, prefill: String, suppData: Bundle?) {
        Timber.tag(TAG).d("onItemReply pos=$position wrapper=$commentWrapper, prefill=$prefill, suppData=$suppData")
    }

    override fun onItemMore(position: Int, commentWrapper: CommentItemWrapperInterface, username: String) {
        Timber.tag(TAG).d("onItemMore pos=$position wrapper=$commentWrapper, username=$username")
    }

    override fun onItemLike(position: Int, commentWrapper: CommentItemWrapperInterface): Boolean {
        Timber.tag(TAG).d("onItemLike pos=$position wrapper=$commentWrapper")
        return false
    }

    override fun onItemDislike(position: Int, commentWrapper: CommentItemWrapperInterface): Boolean {
        Timber.tag(TAG).d("onItemDislike pos=$position wrapper=$commentWrapper")
        return false
    }

    override fun onItemUnlike(position: Int, commentWrapper: CommentItemWrapperInterface): Boolean {
        Timber.tag(TAG).d("onItemUnlike pos=$position wrapper=$commentWrapper")
        return false
    }

    override fun onAuthorNameClick(authorName: String, commentWrapper: CommentItemWrapperInterface) {
        Timber.tag(TAG).d("onAuthorNameClick authrname=$authorName")
    }

    override fun onAuthorAvatarClick(authorName: String) {
        Timber.tag(TAG).d("onAuthorAvatarClick authrname=$authorName")
    }

    override fun onMentionedUserClick(username: String, accountId: String) {
        Timber.tag(TAG).d("onMentionedUserClick mentionedUserClick=$username")
    }

    override fun onLongClickMedia(position: Int, commentWrapper: CommentItemWrapperInterface): Boolean {
        Timber.tag(TAG).d("onLongClickMedia position=$position, wrapper=$commentWrapper")
        return false
    }

    override fun onItemOpenOverlay(commentWrapper: CommentItemWrapperInterface, uivAdapter: UIVAdapter) {
        Timber.tag(TAG).d("onItemOpenOverlay wrapper=$commentWrapper, uivAdapter=$uivAdapter")
    }

    override fun onItemReport(position: Int, commentWrapper: CommentItemWrapperInterface) {
        Timber.tag(TAG).d("onItemReport position=$position,  wrapper=$commentWrapper")
    }

    override fun onItemReportAccepted(commentWrapper: CommentItemWrapperInterface) {
        Timber.tag(TAG).d("onItemReportAccepted position=$commentWrapper")
    }

    override fun onItemHide(position: Int, commentWrapper: CommentItemWrapperInterface) {
        Timber.tag(TAG).d("onItemHide position=$position, wrapper=$commentWrapper")
    }

    override fun onItemDelete(position: Int, commentWrapper: CommentItemWrapperInterface) {
        Timber.tag(TAG).d("onItemDelete position=$position, wrapper=$commentWrapper")
    }

    override fun onItemDeleteAccpeted(position: Int, commentWrapper: CommentItemWrapperInterface) {
        Timber.tag(TAG).d("onItemDeleteAccepted position=$position, wrapper=$commentWrapper")
    }

    override fun onItemCopy(commentWrapper: CommentItemWrapperInterface) {
        Timber.tag(TAG).d("onItemCopy wrapper=$commentWrapper")
    }

    override fun onItemShare(commentWrapper: CommentItemWrapperInterface) {
        Timber.tag(TAG).d("onItemShare wrapper=$commentWrapper")
    }

    override fun onItemProBadge(commentWrapper: CommentItemWrapperInterface) {
        Timber.tag(TAG).d("onItemProBadge wrapper=$commentWrapper")
    }

    override fun onRefresh(position: Int, commentWrapper: CommentItemWrapperInterface) {
        Timber.tag(TAG).d("onItemRefresh position=$position, wrapper=$commentWrapper")
    }

    override fun onItemUrl(commentItemWrapper: CommentItemWrapperInterface, url: String) {
        Timber.tag(TAG).d("onItemUrl wrapper=$commentItemWrapper, url=$url")
    }

    override fun onOpenThread(commentItemWrapperInterface: CommentItemWrapperInterface) {
        Timber.tag(TAG).d("onOpenThread wrapper=$commentItemWrapperInterface")
    }

    override fun onLoadMoreContainerClick(position: Int, commentWrapper: CommentItemWrapperInterface) {
        Timber.tag(TAG).d("onLoadMoreContainerClick wrapper=$commentWrapper")
    }

    override fun onLoadPrevContainerClick(position: Int, commentWrapper: CommentItemWrapperInterface) {
        Timber.tag(TAG).d("onLoadPrevContainerClick wrapper=$commentWrapper")
    }

    override fun onItemMediaSave(position: Int, commentWrapper: CommentItemWrapperInterface) {
        Timber.tag(TAG).d("onItemMediaSave wrapper=$commentWrapper")
    }
}
