package com.under9.android.comments.adapter

import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import com.ninegag.android.blitz2.adapter.BaseToggleableAdapter
import com.under9.android.commentsystem.R

class CommentListHeaderAdapter(
        private val clickListener: View.OnClickListener
) : BaseToggleableAdapter() {

    class HeaderModel(val leftText: Int, val rightText: Int, val leftActionId: Int, val rightActionId: Int)
    internal class VH(v: View, clickListener: View.OnClickListener): ViewHolder(v) {
        val rightActionButton: TextView = v.findViewById(R.id.comment_action_right)
        val leftActionButton: TextView = v.findViewById(R.id.comment_action_left)

        init {
            leftActionButton.setOnClickListener(clickListener)
            rightActionButton.setOnClickListener(clickListener)
        }
    }

    val sortHotCommentHeaderModel: HeaderModel by lazy(LazyThreadSafetyMode.NONE) {
        HeaderModel(R.string.header_hot_comments, -1, R.id.action_sort_comment, -1)
    }

    val sortNewCommentHeaderModel: HeaderModel by lazy(LazyThreadSafetyMode.NONE) {
        HeaderModel(R.string.header_new_comments, -1, R.id.action_sort_comment, -1)
    }

    val viewAllCommentsHeaderModel: HeaderModel by lazy(LazyThreadSafetyMode.NONE) {
        HeaderModel(R.string.header_single_thread, R.string.header_view_all_comments, -1, R.id.action_view_all_comments)
    }

    val defaultHeaderDisplayStrategy by lazy(LazyThreadSafetyMode.NONE) {
        DefaultHeaderDisplayStrategy()
    }

    val boardPostHeaderDisplayStrategy by lazy {
        BoardPostHeaderDisplayStrategy()
    }

    lateinit var headerDisplayStrategy: HeaderDisplayStrategy
    var headerModel: HeaderModel = sortHotCommentHeaderModel

    init {
        isEnable = true
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
        val v = LayoutInflater.from(parent.context).inflate(R.layout.comment_system_header, parent, false)
        return VH(v, clickListener)
    }

    override fun onBindViewHolder(vh: ViewHolder, i: Int) {
        super.onBindViewHolder(vh, i)
        (vh as VH).apply {
            if (headerModel.leftText == -1) {
                leftActionButton.visibility = View.GONE
            } else {
                leftActionButton.visibility = View.VISIBLE
                leftActionButton.setText(headerModel.leftText)
            }

            if (headerModel.rightText == -1) {
                rightActionButton.visibility = View.GONE
            } else {
                rightActionButton.visibility = View.VISIBLE
                rightActionButton.setText(headerModel.rightText)
            }

            leftActionButton.tag = headerModel.leftActionId
            rightActionButton.tag = headerModel.rightActionId
        }
    }

    interface HeaderDisplayStrategy {
        operator fun invoke(commentListHeaderAdapter: CommentListHeaderAdapter, level: Int, loadType: Int)
    }

    class DefaultHeaderDisplayStrategy : HeaderDisplayStrategy {
        override fun invoke(commentListHeaderAdapter: CommentListHeaderAdapter, level: Int, loadType: Int) {
            with(commentListHeaderAdapter) {
                if (level > 1) {
                    headerModel = viewAllCommentsHeaderModel
                } else {
                    if (loadType == com.under9.android.comments.data.query.comment.CommentListQueryParam2.LOAD_TYPE_HOT) {
                        headerModel = sortHotCommentHeaderModel
                    } else if (loadType == com.under9.android.comments.data.query.comment.CommentListQueryParam2.LOAD_TYPE_NEW) {
                        headerModel = sortNewCommentHeaderModel
                    }
                }
            }
        }
    }

    class BoardPostHeaderDisplayStrategy : HeaderDisplayStrategy {
        override fun invoke(commentListHeaderAdapter: CommentListHeaderAdapter, level: Int, loadType: Int) {
            with(commentListHeaderAdapter) {
                if (level > 1) {
                    headerModel = viewAllCommentsHeaderModel
                    commentListHeaderAdapter.isEnable = true
                } else {
                    commentListHeaderAdapter.isEnable = false
                }
            }
        }

    }
}