package com.under9.android.comments.adapter

import android.os.Bundle
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.RecyclerView
import com.ninegag.android.blitz2.adapter.BlitzItemAdapter
import com.under9.android.comments.CommentSystem
import com.under9.android.comments.controller.RepositoryManager
import com.under9.android.comments.data.query.comment.CommentListQueryResult2
import com.under9.android.comments.model.ui.CommentItemThemeAttr
import com.under9.android.comments.model.wrapper.CommentItemWrapperInterface
import com.under9.android.comments.model.wrapper.CommentListItemWrapper
import com.under9.android.comments.ui.renderer.CommentItemRendererDelegateV2
import com.under9.android.comments.ui.widget.CommentItemView

class CommentListItemAdapter(
        commentListItemWrapper: CommentListItemWrapper,
        renderOptions: Bundle?,
        commentItemActionHandler: CommentItemAction
) : BlitzItemAdapter<CommentItemWrapperInterface>(commentListItemWrapper.list) {

    internal class VH(view: View): ViewHolder(view)

    /**
     * @see CommentListQueryResult2.level
     */
    var level: Int = 1
    var selectedCommentId: String? = null
    set(value) {
        field = value
    }

    private val themeAttr = CommentItemThemeAttr()
    private val commentItemRendererDelegate = CommentItemRendererDelegateV2(
            CommentSystem.getInstance().config.domainMapperModule,
            RepositoryManager.appInfoRepository,
            renderOptions,
            commentItemActionHandler
    )

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        val v = CommentItemView(parent.context)
        v.layoutParams = ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT)
        v.commentItemThemeAttr = themeAttr
        themeAttr.mayInitThemeAttr(parent.context)
        return VH(v)
    }

    override fun onBindViewHolder(vh: ViewHolder, i: Int) {
        super.onBindViewHolder(vh, i)
        commentItemRendererDelegate.bind(level, getItem(i), themeAttr, vh, i)
    }

    override fun getItemCount(): Int {
        return super.getItemCount()
    }

    fun refreshRenderOptions(renderOptions: Bundle?) {
        commentItemRendererDelegate.refreshRenderOptions(renderOptions)
    }
}