package com.under9.android.comments.adapter

import android.graphics.Color
import android.graphics.Typeface
import android.view.Gravity
import android.view.View
import android.view.ViewGroup
import android.widget.AbsListView
import android.widget.LinearLayout
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import com.ninegag.android.blitz2.adapter.BlitzSubAdapter
import com.under9.android.commentsystem.R
import com.under9.android.lib.util.UiUtil

class EmptyViewRenderer {
    var show = false
    var placeholderText = R.string.no_comments
    var tvId = View.generateViewId()

    fun onCreateViewHolder(viewGroup: ViewGroup, viewType: Int): BlitzSubAdapter.ViewHolder {
        val context = viewGroup.context
        val wrapperLayout = LinearLayout(context)
        val wrapperLp = AbsListView.LayoutParams(AbsListView.LayoutParams.MATCH_PARENT, AbsListView.LayoutParams.MATCH_PARENT)
        wrapperLayout.layoutParams = wrapperLp
        wrapperLayout.gravity = Gravity.CENTER

        val ll = LinearLayout(context)
        val llLp = LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT)
        ll.orientation = LinearLayout.VERTICAL

        val llEmptyView = LinearLayout(context)
        llEmptyView.layoutParams = llLp
        llEmptyView.gravity = Gravity.CENTER
        llEmptyView.id = R.id.emptyView

        val tv = TextView(context)
        val tvLp = LinearLayout.LayoutParams(LinearLayout.LayoutParams.WRAP_CONTENT, LinearLayout.LayoutParams.WRAP_CONTENT)
        tvLp.gravity = Gravity.CENTER
        tv.id = tvId
        tv.setPadding(
                0,
                UiUtil.dpToPx(context, 80),
                0,
                UiUtil.dpToPx(context, 120)
        )
        tv.textSize = 14f
        tv.setTypeface(tv.typeface, Typeface.BOLD)
        tv.setTextColor(Color.parseColor("#FF999999"))
        tv.text = context.getString(placeholderText)

        llEmptyView.addView(tv, tvLp)
        ll.addView(llEmptyView, llLp)

        wrapperLayout.addView(ll)

        return VH(wrapperLayout, llEmptyView, tv)
    }

    fun onBindViewHolder(viewHolder: RecyclerView.ViewHolder, position: Int) {
        val vh = viewHolder as VH
        if (show) {
            vh.emptyView.visibility = View.VISIBLE
            vh.itemView.visibility = View.VISIBLE
            vh.tv.text = vh.itemView.context.getString(placeholderText)
        } else {
            vh.itemView.visibility = View.GONE
            vh.emptyView.visibility = View.GONE
        }
    }

    internal class VH(itemView: View, emptyView: View, tv: TextView)
        : BlitzSubAdapter.ViewHolder(itemView) {

        internal val emptyView = emptyView
        internal val tv = tv
    }

}