package com.under9.android.comments.api

import com.google.gson.Gson
import com.under9.android.comments.model.api.ApiBaseIOException
import com.under9.android.comments.model.api.ApiResponse
import okhttp3.RequestBody
import okhttp3.ResponseBody
import retrofit2.Converter
import retrofit2.Retrofit
import retrofit2.converter.gson.GsonConverterFactory
import java.io.IOException
import java.lang.reflect.Type

class ResponseBodyConverterFactory(gson: Gson) : Converter.Factory() {
    private val gsonConverterFactory: GsonConverterFactory = GsonConverterFactory.create(gson)

    override fun responseBodyConverter(type: Type, annotations: Array<Annotation>, retrofit: Retrofit): Converter<ResponseBody, *>? {
        val gsonConverter = gsonConverterFactory.responseBodyConverter(type, annotations, retrofit)
        return ResponseBodyConverter(gsonConverter as Converter<ResponseBody, *>)
    }

    override fun requestBodyConverter(type: Type?, parameterAnnotations: Array<Annotation>,
                                      methodAnnotations: Array<Annotation>, retrofit: Retrofit): Converter<*, RequestBody>? {
        return gsonConverterFactory.requestBodyConverter(type!!, parameterAnnotations, methodAnnotations, retrofit)
    }
}

class ResponseBodyConverter(private val converter: Converter<ResponseBody, *>) : Converter<ResponseBody, Any> {

    @Throws(IOException::class)
    override fun convert(responseBody: ResponseBody): Any {
        val response = converter.convert(responseBody)
        if (response is ApiResponse) {
            return if (response.isOK) {
                response
            } else {
                throw ApiBaseIOException(response.error)
            }
        } else {
            throw IOException(response.toString())
        }
    }
}