package com.under9.android.comments.controller

import com.under9.android.comments.CommentSystem
import com.under9.android.comments.api.CommentApi
import com.under9.android.comments.GsonFactory
import com.under9.android.comments.data.repository.*

object RepositoryManager {
    private val dataController = DataController.getInstance()
    private val commentSystem = CommentSystem.getInstance()

    @JvmStatic
    val userRepository: UserRepositoryInterface by lazy {
        UserRepository(dataController, CommentApi.instance.apiService,
                GsonFactory.getGson(), commentSystem, localCommentListRepository)
    }

    @JvmStatic
    val appInfoRepository: AppInfoRepositoryInterface by lazy {
        AppInfoRepository(dataController, CommentApi.instance.cdnApiService)
    }

    @JvmStatic
    val commentListRepository: CommentListRepositoryInterface by lazy {
        CommentListRepository(dataController, CommentApi.instance.apiService, localUserRepository, localCommentListRepository)
    }

    @JvmStatic
    val cacheableCommentListRepository: CommentListRepositoryInterface by lazy {
        CacheableCommentListRepository(dataController, CommentApi.instance.cdnApiService, localUserRepository, localCommentListRepository)
    }

    @JvmStatic
    val localCommentListRepository: LocalCommentListRepositoryInterface by lazy {
        LocalCommentListRepository(dataController, localUserRepository)
    }

    @JvmStatic
    val localUserRepository: LocalUserRepositoryInterface by lazy {
        LocalUserRepository(dataController)
    }

    /**
     * Determine if a cacheable version should be got
     * @param url
     * @return
     */
    @JvmStatic
    fun getCommentListRepositoryByUrl(url: String): CommentListRepositoryInterface {
        return if (userRepository.isPostCommented(url) || userRepository.isPostReported(url)) {
            commentListRepository
        } else {
            cacheableCommentListRepository
        }
    }
}
