package com.under9.android.comments.data.processor

import com.under9.android.comments.controller.DataController
import com.under9.android.comments.data.query.comment.CommentListQueryParam2
import com.under9.android.comments.data.query.comment.CommentListQueryResult2
import com.under9.android.comments.data.repository.LocalCommentListRepositoryInterface
import com.under9.android.comments.data.repository.LocalUserRepositoryInterface
import com.under9.android.comments.model.api.ApiCommentList2
import io.reactivex.Flowable
import timber.log.Timber

class CommentListNextProcessor(
        dataController: DataController,
        private val queryParam: CommentListQueryParam2,
        private val localUserRepository: LocalUserRepositoryInterface,
        private val localCommentListRepository: LocalCommentListRepositoryInterface
): BaseApiResponseProcessor<ApiCommentList2>(dataController) {

    private val listKey = queryParam.listKey
    private val localNextOffset = queryParam.localNextOffset
    private val direction = queryParam.direction
    private val isRefresh = queryParam.isRefresh

    override fun process(apiResponse: ApiCommentList2): Flowable<CommentListQueryResult2> {
        val prev = apiResponse.payload.prev
        val next = apiResponse.payload.next
        val comments = apiResponse.payload.comments
        val opUserId = apiResponse.payload.opUserId
        val level = apiResponse.payload.level
        var startingOrder = localCommentListRepository.getMaxLocalInsertOrder(listKey)

        comments.forEach { apiComment ->
            val user = localUserRepository.obtainUser(apiComment.user)
            if (user != null) {
                Timber.d("id=${apiComment.commentId}, text=${apiComment.text}, permalink=${apiComment.permalink}")
                localCommentListRepository.appendList(++startingOrder, listKey, queryParam.url, apiComment, user)
            }
        }

        val items = localCommentListRepository.getNextListItems(listKey, localNextOffset, comments.size + (level - 1))
        val listMeta = localCommentListRepository.getListMeta(listKey, direction)

        return Flowable.just(CommentListQueryResult2(
                listMeta[CommentListQueryResult2.KEY_PREV],
                listMeta[CommentListQueryResult2.KEY_NEXT], opUserId, level, items))
    }
}