package com.under9.android.comments.data.processor

import com.under9.android.comments.controller.DataController
import com.under9.android.comments.data.query.comment.CommentListQueryParam2
import com.under9.android.comments.data.query.comment.CommentListQueryResult2
import com.under9.android.comments.data.repository.LocalCommentListRepositoryInterface
import com.under9.android.comments.data.repository.LocalUserRepositoryInterface
import com.under9.android.comments.model.api.ApiCommentList2
import io.reactivex.Flowable
import timber.log.Timber

class CommentListProcessor2(
        dataController: DataController,
        private val queryParam: CommentListQueryParam2,
        private val localUserRepository: LocalUserRepositoryInterface,
        private val localCommentListRepository: LocalCommentListRepositoryInterface
) : BaseApiResponseProcessor<ApiCommentList2>(dataController) {

    private val nextProcessor: CommentListNextProcessor by lazy(LazyThreadSafetyMode.NONE) {
        CommentListNextProcessor(dataController, queryParam, localUserRepository, localCommentListRepository)
    }

    private val prevProcessor: CommentListPrevProcessor by lazy(LazyThreadSafetyMode.NONE) {
        CommentListPrevProcessor(dataController, queryParam, localUserRepository, localCommentListRepository)
    }

    internal val listKey = queryParam.listKey
    internal val localNextOffset = queryParam.localNextOffset
    internal val direction = queryParam.direction
    internal val isRefresh = queryParam.isRefresh

    override fun process(apiResponse: ApiCommentList2): Flowable<CommentListQueryResult2> {
        if (queryParam.isRefresh) {
            localCommentListRepository.clearList(listKey)
        }

        val prev = apiResponse.payload.prev
        val next = apiResponse.payload.next
        val parentComment = apiResponse.payload.parent
        val opUserId = apiResponse.payload.opUserId
        val level = apiResponse.payload.level

        if (queryParam.isRefresh) {
            localCommentListRepository.updateListByListKey(listKey, prev, next, opUserId, level, direction, System.currentTimeMillis())
        } else {
            localCommentListRepository.updateListByListKey(listKey, prev, next, opUserId, level, direction, -1)
        }

        parentComment?.run {
            Timber.d("parent, id=${this.commentId}, text=${this.text}, permalink=$permalink")
            val user = localUserRepository.obtainUser(user) ?: return@run
            localCommentListRepository.replaceParent(listKey, queryParam.url, this, user)
        }

        return if (direction == CommentListQueryParam2.DIRECTION_NEXT) {
            nextProcessor.process(apiResponse)
        } else {
            prevProcessor.process(apiResponse)
        }
    }
}