package com.under9.android.comments.data.repository

import com.under9.android.comments.model.CommentItem
import com.under9.android.comments.model.CommentList
import com.under9.android.comments.model.CommentListItem2
import com.under9.android.comments.model.User
import com.under9.android.comments.model.api.ApiComment2

interface LocalCommentListRepositoryInterface {
    /**
     * Get current list's listKey
     */
    fun obtainListByListKey(listKey: String): CommentList

    /**
     * Update listKey's values
     *
     * @param listKey
     * @param prevCursor
     * @param nextCursor
     */
    fun updateListByListKey(listKey: String, prevCursor: String?,
                            nextCursor: String?, opUserId: String, level: Int, direction: Int, lastRefreshTs: Long)

    /**
     * @param listKey
     */
    fun getCommentListItem(listKey: String): List<CommentListItem2>

    fun getCommentItemById(commentId: String): CommentItem?

    /**
     * @param ids Comment ids
     */
    fun getCommentItemsByIds(ids: Collection<String>): List<CommentItem>

    fun getLatestComment(listKey: String): CommentItem?

    /**
     *
     */
    fun replaceCommentItem(url: String, item: ApiComment2, commentUser: User): CommentItem

    /**
     * Replace a pending comment with a the one obtained from API
     */
    fun replacePendingCommentItem(url: String, id: Long, item: ApiComment2, commentUser: User): CommentItem?

    fun appendList(startingLocalIndex: Long, listKey: String, url: String, item: ApiComment2, commentUser: User): Boolean

    fun prependList(startingLocalIndex: Long, listKey: String, url: String, item: ApiComment2, commentUser: User): Boolean

    fun getMaxLocalInsertOrder(listKey: String): Long

    fun updateLocalInsertOrder(listKey: String, startingOrder: Int, prependSize: Int)

    fun getNextListItems(listKey: String, offset: Int, limit: Int): List<CommentListItem2>

    fun replaceParent(listKey: String, url: String, item: ApiComment2, commentUser: User)

    fun getListMeta(listKey: String, direction: Int): Map<String, String?>

    fun clearList(listKey: String)

    fun updateCommentItems(commentItems: List<CommentItem>)

    fun createPendingCommentItem(url: String, listKey: String, level: Int,
                                 content: String, parent: String?, sourceMetaJson: String?, sourceMetaHash: String?): CommentListItem2

    fun getCommentListItemByDbId(id: Long): CommentListItem2?
    
    fun getCommentListItemByCommentDbId(id: Long): List<CommentListItem2>

    fun clearPendingItemByDbId(id: Long)

    fun clear()
}