package com.under9.android.comments.model.api

import androidx.collection.ArrayMap
import com.google.gson.JsonDeserializationContext
import com.google.gson.JsonElement
import com.google.gson.JsonParseException
import com.google.gson.reflect.TypeToken
import com.ninegag.android.app.model.ApiPrimitiveTypeCheckDeserializer
import com.under9.android.comments.GsonFactory
import com.under9.android.comments.controller.MetricsController
import com.under9.android.lib.util.GsonUtil
import timber.log.Timber
import java.lang.Exception
import java.lang.reflect.Type

data class ApiComment2(
        @JvmField var commentId: String,
        @JvmField var level: Int,
        @JvmField var parent: String?,
        @JvmField var permalink: String,
        @JvmField var text: String,
        @JvmField var timestamp: Long,
        @JvmField var type: String,
        @JvmField var isVoteMasked: Int,
        @JvmField var mediaText: String,
        @JvmField var user: ApiUser,
        @JvmField var likeCount: Int,
        @JvmField var dislikeCount: Int,
        @JvmField var isPinned: Int,
        @JvmField var childrenTotal: Int,
        @JvmField var childrenUrl: String?,
        @JvmField var mentionMapping: ArrayMap<String, String>?,
        @JvmField var media: ArrayList<MediaData>?
): ApiResponse() {

    class ApiComment2Deserializer : ApiPrimitiveTypeCheckDeserializer<ApiComment2>() {
        override fun deserialize(el: JsonElement?, typeOfT: Type?, context: JsonDeserializationContext?): ApiComment2? {
            if (el?.isJsonObject != true) {
                MetricsController.logApiUnexpectedDataTypeInvalidJSONObject(el.toString())
                return null
            }

            try {
                val obj = el.asJsonObject
                val gson = GsonFactory.getGson()

                val media: ArrayList<MediaData>? = if (obj.has("media")) {
                    val mediaData = gson.fromJson(getArrayElementIfExist(obj, "media"), Array<MediaData>::class.java)
                    val list = ArrayList<MediaData>()
                    for (mediaDatum in mediaData) {
                        list.add(mediaDatum)
                    }
                    list
                } else {
                    null
                }

                val childrenUrl = if (obj.has("childrenUrl") && !obj.get("childrenUrl").isJsonNull) {
                    getStringElement(obj, "childrenUrl")
                } else {
                    null
                }


                return ApiComment2(
                        commentId = getStringElement(obj, "commentId"),
                        level = getIntElement(obj, "level"),
                        parent = getStringElementIfExist(obj, "parent"),
                        permalink = getStringElement(obj, "permalink"),
                        text = getStringElement(obj, "text"),
                        mediaText = getStringElement(obj, "mediaText"),
                        timestamp = getLongElement(obj, "timestamp"),
                        type = getStringElement(obj, "type"),
                        isVoteMasked = getIntElement(obj, "isVoteMasked"),
                        user = gson.fromJson(getObjectElement(obj, "user"), ApiUser::class.java),
                        likeCount = getIntElement(obj, "likeCount"),
                        dislikeCount = getIntElement(obj, "dislikeCount"),
                        isPinned = getIntElement(obj, "isPinned"),
                        childrenTotal = getIntElement(obj, "childrenTotal"),
                        childrenUrl = childrenUrl,
                        mentionMapping = gson.fromJson(getObjectElementIfExist(obj, "mentionMapping"), GsonUtil.getArrayMapStringStringType()),
                        media = media
                )
            } catch (e: JsonParseException) {
                MetricsController.logApiUnexpectedDataType(e.message, el.toString())
                return null
            }
        }

    }
}