package com.under9.android.comments.model.api

import com.google.gson.JsonDeserializationContext
import com.google.gson.JsonElement
import com.google.gson.JsonParseException
import com.google.gson.reflect.TypeToken
import com.ninegag.android.app.model.ApiPrimitiveTypeCheckDeserializer
import com.under9.android.comments.GsonFactory
import com.under9.android.comments.controller.MetricsController
import java.lang.reflect.Type

data class ApiCommentList2(
        @JvmField var payload: Payload
): ApiResponse() {

    companion object {
        val listApiComment2Type: Type by lazy(LazyThreadSafetyMode.NONE) {
            object: TypeToken<ArrayList<ApiComment2>>() {}.type
        }

        val apiComment2Type: Type by lazy(LazyThreadSafetyMode.NONE) {
            object: TypeToken<ApiComment2>() {}.type
        }
    }

    data class Payload(
            @JvmField var url: String,
            @JvmField var status: String?,
            @JvmField var lock: Boolean,
            @JvmField var total: Int,
            @JvmField var opUserId: String,
            @JvmField var comments: ArrayList<ApiComment2>,
            @JvmField var prev: String?,
            @JvmField var next: String?,
            @JvmField var parent: ApiComment2?,
            @JvmField var level: Int
    )

    class ApiCommentList2PayloadDeserializer : ApiPrimitiveTypeCheckDeserializer<Payload>() {
        override fun deserialize(el: JsonElement?, typeOfT: Type?, context: JsonDeserializationContext?): Payload? {
            if (el?.isJsonObject != true) {
                MetricsController.logApiUnexpectedDataTypeInvalidJSONObject(el.toString())
                return null
            }

            try {
                val obj = el.asJsonObject
                val gson = GsonFactory.getGson()

                val parentElem = obj.get("parent")
                val prevElem = obj.get("prev")
                val nextElem = obj.get("next")
                val statusElem = obj.get("status")
                val parent: ApiComment2? = if (parentElem == null || parentElem.isJsonNull) null else gson.fromJson(parentElem, apiComment2Type)
                val prev: String? = if (prevElem == null || prevElem.isJsonNull) null else gson.fromJson(prevElem, String::class.java)
                val next: String? = if (nextElem == null || nextElem.isJsonNull) null else gson.fromJson(nextElem, String::class.java)
                val status: String? = if (statusElem == null || statusElem.isJsonNull) null else gson.fromJson(statusElem, String::class.java)

                return Payload(
                        url = getStringElement(obj, "url"),
                        status = status,
                        lock = getBooleanElement(obj, "lock"),
                        total = getIntElement(obj, "total"),
                        opUserId = getStringElement(obj, "opUserId"),
                        comments = gson.fromJson(getArrayElementIfExist(obj, "comments"), listApiComment2Type),
                        prev = prev,
                        next = next,
                        parent = parent,
                        level = getIntElement(obj, "level")
                )
            } catch (e: JsonParseException) {
                MetricsController.logApiUnexpectedDataType(e.message, el.toString())
                return null
            }
        }

    }
}