package com.under9.android.comments.model.wrapper

import com.jakewharton.rxrelay2.BehaviorRelay
import com.ninegag.android.blitz2.Blitz
import com.ninegag.android.blitz2.PagingInfo
import com.ninegag.android.blitz2.RxBlitz
import com.under9.android.comments.data.query.comment.CommentListQueryParam2
import com.under9.android.comments.data.repository.*
import timber.log.Timber

class CommentListItemWrapper(
        localCommentListRepository: LocalCommentListRepositoryInterface,
        commentListRepo: CommentListRepositoryInterface,
        userRepository: UserRepositoryInterface
) {

    private val dataSource: CommentDataSource = CommentDataSource(localCommentListRepository, commentListRepo, userRepository)
    val list: RxBlitz<CommentItemWrapperInterface, String, CommentListQueryParam2> = RxBlitz(dataSource)

    var url: String = ""
        set(value) {
            field = value
            dataSource.url = value
        }
    var commentId: String? = ""
        set(value) {
            field = value
            dataSource.commentId = value
        }
    var commentChildrenUrl: String? = ""
        set(value) {
            field = value
            dataSource.commentChildrenUrl = value
        }
    var loadType: Int = CommentListQueryParam2.LOAD_TYPE_HOT
        set(value) {
            field = value
            dataSource.loadType = value
        }

    fun listKey(): String? = dataSource.listKey

    fun initLoad() {
        dataSource.direction = CommentListQueryParam2.DIRECTION_NEXT
        list.initLoad()
    }

    fun loadNext(): Boolean {
        if (list.hasNext()) {
            Timber.d("loadNext=${dataSource.url}")
            dataSource.direction = CommentListQueryParam2.DIRECTION_NEXT
            list.loadNext()
            return true
        }
        return false
    }

    fun loadPrev(): Boolean {
        if (list.hasPrev()) {
            Timber.d("loadPrev=${dataSource.url}")
            dataSource.direction = CommentListQueryParam2.DIRECTION_PREV
            list.loadPrev()
            return true
        }
        return false
    }

    fun remoteRefresh() {
        list.remoteRefresh()
    }

    fun size(): Int = list.size

    fun listState(): BehaviorRelay<Int> = list.listState

    fun errorState(): BehaviorRelay<Throwable> = list.errorState

    fun addListener(listener: Blitz.LoaderListener<CommentItemWrapperInterface>) = list.addListener(listener)
}