package com.under9.android.comments.model.wrapper

import com.under9.android.comments.CommentSystem
import com.under9.android.comments.model.api.ImageMetaByType
import com.under9.android.comments.model.constant.CommentConstant
import com.under9.android.lib.widget.media.overlayv3.IOverlayable
import com.under9.android.lib.widget.media.wrapper.IMediaWrapperV2

object CommentTransformer {

    @JvmStatic
    fun transform(commentWrapper: CommentWrapperInterface): IMediaWrapperV2? {

        if(commentWrapper.getEmbedMediaMeta() == null) return null

        val embedImage = parseEmbedMedia(commentWrapper) ?: return null

        with(embedImage) {
            if (image == null || type == null) return null

            when (type) {
                com.under9.android.comments.model.constant.CommentConstant.MEDIA_TYPE_STATIC -> {
                    val enabledHD = commentWrapper.isEnabledHD()
                    val imageUrl = if (!enabledHD) {
                        embedImage.image!!.getImageUrl()
                    } else {
                        embedImage.imageXLarge!!.url
                    }

                    val width = if (!enabledHD) {
                        embedImage.image!!.width
                    } else {
                        embedImage.imageXLarge!!.width
                    }

                    val height = if (!enabledHD) {
                        embedImage.image!!.height
                    } else {
                       embedImage.imageXLarge!!.height
                    }
                    
                    return CommentTransformWrapper(
                            mediaHeight = width,
                            mediaWidth = height,
                            mediaId = commentWrapper.getCommentId(),
                            mediaTitle = commentWrapper.getTime().toString(),
                            otherVideo = !this.isAnimated,
                            commentType = if (type == CommentConstant.MEDIA_TYPE_ANIMATED) IMediaWrapperV2.MEDIA_TYPE_ANIMATED else IMediaWrapperV2.MEDIA_TYPE_STATIC,
                            mediaImageUrl = imageUrl,
                            mediaVideoUrl = null,
                            isEnabledHD = commentWrapper.isEnabledHD())
                }
                com.under9.android.comments.model.constant.CommentConstant.MEDIA_TYPE_ANIMATED -> {
                    return CommentTransformWrapper(
                            mediaHeight = image!!.height,
                            mediaWidth = image!!.width,
                            mediaId = commentWrapper.getCommentId(),
                            mediaTitle = commentWrapper.getTime().toString(),
                            otherVideo = !this.isAnimated && embedImage.mp4Url != null,
                            commentType = if (type == CommentConstant.MEDIA_TYPE_ANIMATED) IMediaWrapperV2.MEDIA_TYPE_ANIMATED else IMediaWrapperV2.MEDIA_TYPE_STATIC,
                            mediaImageUrl = null,
                            mediaVideoUrl = embedImage.mp4Url,
                            isEnabledHD = commentWrapper.isEnabledHD())
                }
                // case when the Comment Constant is progressing
                else -> return null
            }
        }

    }

    @JvmStatic
    fun parseEmbedMedia(commentWrapper: CommentWrapperInterface): EmbedImage? {
        return commentWrapper.getEmbedMediaMeta()?.embedImage?.let {
            it
        } ?: run {
            commentWrapper.getFirstMediaEmbedImage()
        } ?: run{
            return null
        }
    }


    fun transform2(commentWrapper: CommentItemWrapperInterface, isEnableHD: Boolean): IMediaWrapperV2? {
        if (commentWrapper.getFirstMedia() == null) return null

        val media = commentWrapper.getFirstMedia()!!.imageMetaByType

        with(media) {
            if (media?.type == null) return null

            when (type) {
                com.under9.android.comments.model.constant.CommentConstant.MEDIA_TYPE_STATIC -> {
                    val imageUrl = if (!isEnableHD) {
                        media.image.getImageUrl()
                    } else {
                        media.imageXLarge.getImageUrl()
                    }

                    val width = if (!isEnableHD) {
                        media.image.width
                    } else {
                        media.imageXLarge.width
                    }

                    val height = if (!isEnableHD) {
                        media.image.height
                    } else {
                        media.imageXLarge.height
                    }

                    return CommentTransformWrapper(
                            mediaHeight = height,
                            mediaWidth = width,
                            mediaId = commentWrapper.getCommentId(),
                            mediaTitle = commentWrapper.getTime().toString(),
                            otherVideo = media.animated == null && media.video != null,
                            commentType = if (type == CommentConstant.MEDIA_TYPE_ANIMATED) IMediaWrapperV2.MEDIA_TYPE_ANIMATED else IMediaWrapperV2.MEDIA_TYPE_STATIC,
                            mediaImageUrl = imageUrl,
                            mediaVideoUrl = null,
                            isEnabledHD = isEnableHD
                    )
                }

                com.under9.android.comments.model.constant.CommentConstant.MEDIA_TYPE_ANIMATED -> {
                    return CommentTransformWrapper(
                            mediaHeight = image!!.height,
                            mediaWidth = image!!.width,
                            mediaId = commentWrapper.getCommentId(),
                            mediaTitle = commentWrapper.getTime().toString(),
                            otherVideo = media.animated == null && media.video != null,
                            commentType = if (type == CommentConstant.MEDIA_TYPE_ANIMATED) IMediaWrapperV2.MEDIA_TYPE_ANIMATED else IMediaWrapperV2.MEDIA_TYPE_STATIC,
                            mediaImageUrl = null,
                            mediaVideoUrl = video.url,
                            isEnabledHD = isEnableHD)
                }
                // case when the Comment Constant is progressing
                else -> return null
            }

        }
    }
}

class CommentTransformWrapper(val mediaVideoUrl : String?,
                              val mediaHeight: Int,
                              var mediaWidth: Int,
                              var mediaTitle: String,
                              var mediaImageUrl: String?,
                              val commentType: String,
                              val otherVideo: Boolean,
                              val mediaId : String, override var isEnabledHD: Boolean) : IMediaWrapperV2, IOverlayable {


    override fun getTitle(): String = mediaTitle

    override fun getId(): String = mediaId

    override fun getImageUrl() = mediaImageUrl

    override fun getImageWidth() = mediaWidth

    override fun getImageHeight() = mediaHeight

    override fun getVideoUrl() = mediaVideoUrl

    // animated is not a gif
    override fun isOtherVideo() = otherVideo

    override fun getMediaType() = commentType
}

