package com.under9.android.comments.model.wrapper

import com.ninegag.android.blitz.BlitzItem
import com.under9.android.comments.CommentSystem
import com.under9.android.comments.model.User
import com.under9.android.lib.util.L10nUtil
import java.lang.ref.WeakReference
import java.util.*

class UserWrapper private constructor(
        underlyingObject: User
): BlitzItem<User>(underlyingObject), UserItemWrapperInterface {
    override fun getUserId(): String = underlyingObject.userId

    override fun getAccountId(): String = underlyingObject.accountId
    
    override fun getName(): String = underlyingObject.displayName

    override fun getEmojiStatus(): String = underlyingObject.emojiStatus

    override fun getCountryEmoji(): String = L10nUtil.countryCodeToEmoji(underlyingObject.country) ?: ""

    override fun getAvatarUrl(): String = underlyingObject.avatarUrl

    override fun isPro(): Boolean = underlyingObject.isActivePro

    override fun isProPlus(): Boolean = underlyingObject.isActiveProPlus

    override fun isActive(): Boolean {
        return (System.currentTimeMillis() - underlyingObject.activeTs * 1000) <= CommentSystem.getInstance().config.activeDuration
    }

    override fun getDisplayName(): String = underlyingObject.displayName

    companion object {
        private val objectMap = WeakHashMap<User, WeakReference<UserWrapper>>()

        @JvmStatic
        fun obtainInstance(user: User): UserItemWrapperInterface {
            synchronized(objectMap) {
                val ref = objectMap[user]
                if (ref != null) {
                    val obj = ref.get()
                    if (obj != null) {
                        return obj
                    }
                }

                val obj = UserWrapper(user)
                objectMap[user] = WeakReference(obj)
                return obj
            }
        }
    }
}