package com.under9.android.comments.ui.renderer

import android.os.Bundle
import androidx.recyclerview.widget.RecyclerView
import com.under9.android.comments.adapter.CommentItemAction
import com.under9.android.comments.adapter.CommentItemClickListenerV2
import com.under9.android.comments.data.repository.AppInfoRepositoryInterface
import com.under9.android.comments.model.ui.CommentItemThemeAttr
import com.under9.android.comments.model.wrapper.CommentItemWrapperInterface
import com.under9.android.lib.network.UrlMapperInterface

class CommentItemRendererDelegateV2(
        private val urlMapperInterface: UrlMapperInterface,
        private val appInfoRepository: AppInfoRepositoryInterface,
        private val renderOptionBundle: Bundle?,
        val commentItemActionHandler: CommentItemAction
): CommentItemInfoRendererInterface {

    /**
     * All UI behaviour keys
     */
    companion object {
        const val KEY_AVATAR_MODE = "avatar_mode"
        const val AVATAR_MODE_IMAGE = 0
        const val AVATAR_MODE_COLOR = 1

        const val KEY_META_TIME_FORMAT = "time_format"
        const val META_TIME_FORMAT_AGO = 0
        const val META_TIME_FORMAT_CHAT = 1

        const val KEY_VOTE_DISPLAY = "vote_display"
        const val VOTE_DISPLAY_HIDE_DOWNVOTE = 0

        const val KEY_SCROLL_TO_FIRST_COMMENT_ON_INIT = "scroll_to_first_comment_on_init"
        const val KEY_ALLOW_SWIPE_REFRESH = "allow_swipe_refresh"
        const val KEY_SHOULD_SHOW_LOAD_PREV_CONTAINER = "should_show_load_prev_container"
        const val KEY_SHOULD_SHOW_LOAD_NEXT_CONTAINER = "should_show_load_next_container"
        const val KEY_SHOULD_SHOW_OP = "should_show_op"
        const val KEY_INDENTATION_LEVEL = "indentation_level"
        const val KEY_SHOULD_HIGHLIGHT_URL = "should_highlight_url"
        const val KEY_SHOULD_USERNAME_CLICKABLE = "should_username_clickable"
        const val KEY_SHOULD_SHOW_COUNTRY = "should_show_country"
        const val KEY_SHOULD_AUTOPLAY = "should_auto_play"
        const val KEY_SHOULD_SHOW_DOWNVOTE_COUNT = "should_show_downvote_count"
        const val KEY_HIGHLIGHT_COMMENT_ID = "highlight_comment_id"
    }

    private val commentItemClickListener: CommentItemClickListenerV2 = CommentItemClickListenerV2(commentItemActionHandler)

    private val renderers: MutableList<BaseCommentItemRenderer> = mutableListOf()

    init {
        renderers.add(BodyRenderer(commentItemClickListener, renderOptionBundle))
        renderers.add(VoteInfoRenderer(commentItemClickListener, renderOptionBundle))
        renderers.add(TextContentRenderer(commentItemClickListener, renderOptionBundle))
        renderers.add(AvatarRenderer(commentItemClickListener, renderOptionBundle))
        renderers.add(UserInfoRenderer(commentItemClickListener, renderOptionBundle))
        renderers.add(OpInfoRenderer(commentItemClickListener, renderOptionBundle))
        renderers.add(MediaRenderer(commentItemClickListener, renderOptionBundle, urlMapperInterface))
    }

    override fun bind(level: Int, wrapper: CommentItemWrapperInterface, themeAttr: CommentItemThemeAttr, viewHolder: RecyclerView.ViewHolder, position: Int) {
        for (i in 0 until renderers.size) {
            renderers[i].bind(level, wrapper, themeAttr, viewHolder, position)
        }
    }

    override fun refreshRenderOptions(renderOptions: Bundle?) {
        for (i in 0 until renderers.size) {
            renderers[i].refreshRenderOptions(renderOptions)
        }
    }
}