package com.under9.android.lib.behavior

import android.app.Activity
import android.content.Context
import android.graphics.Paint
import android.graphics.PorterDuff
import android.graphics.PorterDuffXfermode
import android.os.Build
import android.view.View
import androidx.core.content.ContextCompat
import com.under9.android.lib.util.UiUtil
import com.under9.android.lib.widget.R

/**
 * Current implemented is BedModeV1, just dim the screen
 * Smart Bed mode (BedModeV2) is include for auto change to dark theme and minimize the system brightness
 * */
class BedModeController(private val context: Context, private val isEnabledPureDark: Boolean, private val isEnabledDarkTheme: Boolean) {
    private var customCanvasView: ICustomCanvasView? = null
    private var shouldChangeSystemNavColor = true

    private val dimPaint = Paint().apply {
        color = ContextCompat.getColor(context, R.color.bed_mode_dim_bg)
        xfermode = PorterDuffXfermode(PorterDuff.Mode.SRC_OVER)
    }

    fun registerCanvasView(customCanvasView: ICustomCanvasView) {
        this.customCanvasView = customCanvasView
    }

    fun disableChangeNavColor () {
        shouldChangeSystemNavColor = false
    }
    fun enableBedMode() {
        customCanvasView?.let {
            it.enableDraw(dimPaint)
            // Do not need to handle pre-lollipop, becuase it will drop after pro+ launch
            if (Build.VERSION.SDK_INT >= 21 && shouldChangeSystemNavColor) {
                setSystemNavigationColor(UiUtil.themeAttributeToColor(R.attr.under9_themeBedModeNavigationColor, context, -1), UiUtil.themeAttributeToColor(R.attr.under9_themeBedModeNavigationColor, context, -1))
            }
            (it as View).invalidate()
        }


        // TODO pending for BedModeV2
//        minimizeBrightness()
//        if (!ObjectManager.getInstance().aoc.enableDarkTheme && !enabledBedMode) {
//            if (context is BaseActivity) {
//                context.uiState.updateTheme(true, true, true)
//            }
//            ObjectManager.getInstance().aoc.enableDarkTheme = true
//        }

    }

    fun disableBedMode() {
        customCanvasView?.let {
          it.removeDraw()
            if (Build.VERSION.SDK_INT >= 21 && shouldChangeSystemNavColor) {
                setSystemNavigationColor(UiUtil.themeAttributeToColor(R.attr.under9_themeColorPrimaryDark, context, -1), UiUtil.themeAttributeToColor(R.attr.under9_themeNavigationBarColor, context, -1))
            }
            (it as View).invalidate()
        }

    }


    private fun setSystemNavigationColor(sColor: Int, nColor: Int) {
        if (isEnabledPureDark && isEnabledDarkTheme) return
        val activity = context as Activity?
        activity?.let {
            with (it.window) {
                val originalSystemUiVisibility = decorView.systemUiVisibility
                val decorView = decorView
                val hasLightStatusBar = originalSystemUiVisibility and View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR == View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR
                val hasLightNavBar = originalSystemUiVisibility and View.SYSTEM_UI_FLAG_LIGHT_NAVIGATION_BAR == View.SYSTEM_UI_FLAG_LIGHT_NAVIGATION_BAR

                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                    statusBarColor = sColor
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O_MR1) {
                        navigationBarColor = nColor
                        if (hasLightNavBar && hasLightStatusBar) {
                            decorView.systemUiVisibility = View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR or View.SYSTEM_UI_FLAG_LIGHT_NAVIGATION_BAR
                        }
                    } else if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                        if (hasLightStatusBar) {
                            decorView.systemUiVisibility = View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR
                        }
                    }
                }
            }
        }

    }
    // TODO pending for BedModeV2
    // This function only take effect in real physical android device,
    // it can not take effect in android emulator.
//    private fun minimizeBrightness() {
//        val allowedPermission = checkWriteSettingsPermission()
//        if (allowedPermission) {
//            // Change the screen brightness change mode to manual.
//            Settings.System.putInt(context.contentResolver, Settings.System.SCREEN_BRIGHTNESS_MODE, Settings.System.SCREEN_BRIGHTNESS_MODE_MANUAL)
//            // Apply the screen brightness value to the system
//            // It will also change the screen brightness for the device.
//            Settings.System.putInt(context.contentResolver, Settings.System.SCREEN_BRIGHTNESS, 0)
//        }
//    }


    // TODO pending for BedModeV2
//    fun checkWriteSettingsPermission(): Boolean{
//        return if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M && context is BaseActivity) {
//             if (!Settings.System.canWrite(context)) {
//                val intent = Intent(Settings.ACTION_MANAGE_WRITE_SETTINGS,
//                        Uri.parse("package:" + context.getPackageName()))
//                context.startActivityForResult(intent, REQ_SETTING_WRITE)
//                false
//            } else true
//        } else false
//    }
}

interface ICustomCanvasView {
    fun enableDraw(paint: Paint)

    fun removeDraw()
}