package com.under9.android.lib.bottomsheet

import android.app.Dialog
import android.os.Build
import android.os.Bundle
import android.util.Log
import android.view.View
import androidx.core.content.ContextCompat
import com.google.android.material.bottomsheet.BottomSheetBehavior
import com.google.android.material.bottomsheet.BottomSheetDialog
import com.google.android.material.bottomsheet.BottomSheetDialogFragment
import com.under9.android.lib.util.UiUtil
import com.under9.android.lib.widget.R
import timber.log.Timber


/**
 * Round corner bottom sheet, should just define style here
 * */
abstract class StyledBottomSheetDialogFragment(bottomSheet: View? = null): BottomSheetDialogFragment(){

    protected var bottomSheetBehavior: BottomSheetBehavior<View>? = null
    protected var isEnabledBedMode: Boolean = false
    private var behaviorCallback: BottomSheetBehavior.BottomSheetCallback = object : BottomSheetBehavior.BottomSheetCallback() {
        override fun onStateChanged(bottomSheet: View, newState: Int) {
            if (newState == BottomSheetBehavior.STATE_HIDDEN) {
                dismiss()
            }
        }

        override fun onSlide(bottomSheet: View, slideOffset: Float) {
        }
    }

    init {
        if (bottomSheet != null) {
            bottomSheetBehavior = BottomSheetBehavior.from<View>(bottomSheet)
            bottomSheetBehavior!!.setBottomSheetCallback(behaviorCallback)
        }
    }

    override fun getTheme() = if (!isEnabledBedMode) {
        R.style.BottomSheetDialogThemeNormal
    } else {
        R.style.BottomSheetDialogThemeBedMode
    }

    override fun onCreateDialog(savedInstanceState: Bundle?): Dialog {
        val dialog = BottomSheetDialog(requireContext(), theme)
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O_MR1) {
                // Manually apply the navigation bar color because of:
                // https://stackoverflow.com/questions/48785266/oneplus-5t-on-oxygen-os-ignores-the-specified-color-of-the-navigation-bar
                if (!isEnabledBedMode) {
                    dialog.window?.navigationBarColor = UiUtil.themeAttributeToColor(R.attr.under9_themeNavigationBarColor, context, -1)
                } else {
                    dialog.window?.navigationBarColor = UiUtil.themeAttributeToColor(R.attr.under9_themeBedModeNavigationColor, context, -1)
                }
            } else {
                dialog.window?.navigationBarColor = ContextCompat.getColor(context!!, R.color.under9_theme_black)
            }
        }
        return dialog
    }

    fun setBehaviorCallback(callback: BottomSheetBehavior.BottomSheetCallback, bottomSheet: View) {
        try {
            bottomSheetBehavior = BottomSheetBehavior.from<View>(bottomSheet)
            bottomSheetBehavior!!.setBottomSheetCallback(callback)
        } catch (e: Exception) {
            Timber.e("bottomSheetBehavior" + e)
        }

    }
}