package com.under9.android.lib.util

import androidx.collection.ArrayMap
import com.google.firebase.perf.FirebasePerformance
import com.google.firebase.perf.metrics.Trace
import timber.log.Timber

object FirebasePerformanceWrapper {
    private val sLock = Any()
    private val sTraceMap = ArrayMap<String, Trace>()

    @JvmStatic fun startTrace(key: String) {
        synchronized(sLock) {
            if (sTraceMap.containsKey(key)) return

            try {
                val trace = FirebasePerformance.startTrace(key)
                sTraceMap[key] = trace
                Timber.d("startTrace: key=$key")
            } catch (e: Exception) {
                Timber.w(e, "startTrace: failed!")
            }

        }
    }

    @JvmStatic fun stopTrace(key: String) {
        synchronized(sLock) {
            if (sTraceMap.containsKey(key)) {
                try {
                    sTraceMap[key]!!.stop()
                    sTraceMap.remove(key)
                    Timber.d("stopTrace: key=$key")
                } catch (e: Exception) {
                    Timber.w(e, "stopTrace: failed!")
                }

            }
        }
    }
}