package com.under9.android.lib.util

import androidx.collection.ArrayMap
import androidx.collection.SparseArrayCompat

import com.google.gson.Gson
import com.google.gson.GsonBuilder
import com.google.gson.JsonSyntaxException
import com.google.gson.reflect.TypeToken
import timber.log.Timber

import java.lang.reflect.Type

object GsonUtil {
    @JvmStatic var gson: Gson? = null
        private set

    private var pureGson: Gson? = null

    private var arrayMapStringStringType: Type? = null

    private val gsonObjects = SparseArrayCompat<Gson>()

    interface GsonBuilderProvider {
        fun createGsonBuilder(): GsonBuilder
    }

    @JvmStatic fun getGson(type: Int, provider: GsonBuilderProvider): Gson {
        if (gsonObjects.get(type) == null) {
            synchronized(gsonObjects) {
                if (gsonObjects.get(type) == null) {
                    gsonObjects.put(type, provider.createGsonBuilder().create())
                }
            }
        }
        return gsonObjects.get(type)!!
    }

    @JvmStatic fun getArrayMapStringStringType(): Type {
        if (arrayMapStringStringType == null) {
            arrayMapStringStringType = object : TypeToken<ArrayMap<String, String>>() {

            }.type
        }
        return arrayMapStringStringType!!
    }

    @JvmStatic fun <T> fromJson(json: String?, classOfT: Class<T>): T? {
        init()
        return try {
            gson!!.fromJson(json, classOfT)
        } catch (e: JsonSyntaxException) {
            //System.out.println(json);
            null
        }

    }

    @JvmStatic fun toJson(obj: Any?): String {
        init()
        return gson!!.toJson(obj)
    }

    private fun init() {
        if (pureGson == null) {
            val b = GsonBuilder()
            pureGson = b.create()
        }
        if (gson == null) gson = pureGson
    }

    /**
     * Determine if it's a JSON
     */
    fun isJson(string: String?): Boolean {
        return try {
            gson!!.fromJson(string, Any::class.java)
            true
        } catch (je: JsonSyntaxException) {
            false
        }
    }

    /**
     * Don't call this directly, creating a wrapper function in your module to call
     * it
     */
    @JvmStatic fun <T> fromJson(json: String?, classOfT: Class<T>, type: Int): T? {
        try {
            val gson = gsonObjects[type]
            if (gson != null) {
                return gson.fromJson(json, classOfT)
            }
        } catch (e: JsonSyntaxException) {
            Timber.w(e)
        }

        return null
    }

    @JvmStatic fun <T> fromJson(json: String?, typeToken: Type, type: Int): T? {
        try {
            val gson = gsonObjects[type]
            if (gson != null) {
                return gson.fromJson<T>(json, typeToken)
            }
        } catch (e: JsonSyntaxException) {
            Timber.w(e)
        }

        return null
    }

    @JvmStatic fun toJson(obj: Any?, type: Int): String? {
        try {
            val gson = gsonObjects[type]
            if (gson != null) {
                return gson.toJson(obj)
            }
        } catch (e: JsonSyntaxException) {
            Timber.w(e)
        }

        return null
    }
}
