package com.under9.android.lib.view.rootview

import android.content.Context
import android.graphics.Canvas
import android.graphics.Paint
import android.util.AttributeSet
import androidx.drawerlayout.widget.DrawerLayout
import com.under9.android.lib.behavior.ICustomCanvasView

/**
 * Use case:
 * - If this view is a root view, this custom view available for us to draw a
 * custom color, such as dim screen
 *
 * - It is not necessary to use this view if this is not a root view
 * */
class GagDrawerLayout @JvmOverloads constructor(context: Context,
                                                attrs: AttributeSet? = null,
                                                defStyleAttr: Int = 0) : DrawerLayout(context, attrs, defStyleAttr), ICustomCanvasView {

    private var paint: Paint? = null
    private var enabledCustomDraw = false

    init {
        init(attrs)
    }

    private fun init(attrs: AttributeSet?) {
    }


    override fun enableDraw(paint: Paint) {
        this.paint = paint
        enabledCustomDraw = true
        setWillNotDraw(false)
    }

    override fun removeDraw() {
        this.paint = null
        enabledCustomDraw = false
        setWillNotDraw(true)
    }

    override fun dispatchDraw(canvas: Canvas?) {
        super.dispatchDraw(canvas)
        if (enabledCustomDraw) {
            paint?.let {
                canvas?.drawRect(0F, 0F, width.toFloat(), height.toFloat(), it)
            }
        }
    }

}