package com.under9.android.lib.view.rootview

import android.annotation.TargetApi
import android.content.Context
import android.graphics.Canvas
import android.graphics.Paint
import android.os.Build
import android.util.AttributeSet
import android.widget.RelativeLayout
import com.under9.android.lib.behavior.ICustomCanvasView

/**
 * Use case:
 * - If this view is a root view, this custom view available for us to draw a
 * custom color, such as dim screen
 *
 * - It is not necessary to use this view if this is not a root view
 * */
class GagRelativeLayout : RelativeLayout, ICustomCanvasView {

    @JvmOverloads
    constructor(
            context: Context,
            attrs: AttributeSet? = null,
            defStyleAttr: Int = 0)
            : super(context, attrs, defStyleAttr) {
        init(attrs)
    }


    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
    constructor(
            context: Context,
            attrs: AttributeSet?,
            defStyleAttr: Int,
            defStyleRes: Int)
            : super(context, attrs, defStyleAttr, defStyleRes) {
        init(attrs)
    }

    private var paint: Paint? = null
    private var enabledCustomDraw = false

    private fun init(attrs: AttributeSet?) {
    }


    override fun enableDraw(paint: Paint) {
        this.paint = paint
        enabledCustomDraw = true
        setWillNotDraw(false)
    }

    override fun removeDraw() {
        this.paint = null
        enabledCustomDraw = false
        setWillNotDraw(true)
    }

    override fun dispatchDraw(canvas: Canvas?) {
        super.dispatchDraw(canvas)
        if (enabledCustomDraw) {
            paint?.let {
                canvas?.drawRect(0F, 0F, width.toFloat(), height.toFloat(), it)
            }
        }
    }

}